# flake8: noqa

# import apis into api package
from passageidentity.openapi_client.api.magic_links_api import MagicLinksApi
from passageidentity.openapi_client.api.tokens_api import TokensApi
from passageidentity.openapi_client.api.user_devices_api import UserDevicesApi
from passageidentity.openapi_client.api.users_api import UsersApi




"""Provides the User class for interacting with the Passage API."""

from __future__ import annotations

from typing import TYPE_CHECKING

from passageidentity.errors import PassageError
from passageidentity.openapi_client.exceptions import ApiException
from passageidentity.openapi_client.models.model404_error import Model404Error

from .openapi_client.api import (
    TokensApi,
    UserDevicesApi,
    UsersApi,
)

if TYPE_CHECKING:
    from .openapi_client.models import (
        CreateUserArgs,
        PassageUser,
        UpdateUserArgs,
        WebAuthnDevices,
    )


class User:
    """User class for handling operations to get and update user information."""

    def __init__(self, app_id: str, request_headers: dict[str, str]) -> None:
        """Initialize the User class with the app ID and request headers."""
        self.app_id = app_id
        self.request_headers = request_headers
        self.users_api = UsersApi()
        self.user_devices_api = UserDevicesApi()
        self.tokens_api = TokensApi()

    def get(self, user_id: str) -> PassageUser:
        """Get a user's object using their user ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            return self.users_api.get_user(self.app_id, user_id, _headers=self.request_headers).user
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def get_by_identifier(self, identifier: str) -> PassageUser:
        """Get a user's object using their user identifier."""
        if not identifier:
            msg = "identifier is required."
            raise ValueError(msg)

        try:
            users = self.users_api.list_paginated_users(
                self.app_id,
                limit=1,
                identifier=identifier.lower(),
                _headers=self.request_headers,
            ).users
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

        if len(users) == 0:
            raise PassageError.from_response_error(
                ApiException(status=404, data=Model404Error(code="user_not_found", error="User not found.")),
            )

        return self.get(users[0].id)

    def activate(self, user_id: str) -> PassageUser:
        """Activate a user using their user ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            return self.users_api.activate_user(self.app_id, user_id, _headers=self.request_headers).user
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def deactivate(self, user_id: str) -> PassageUser:
        """Deactivate a user using their user ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            return self.users_api.deactivate_user(self.app_id, user_id, _headers=self.request_headers).user
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def update(self, user_id: str, options: UpdateUserArgs) -> PassageUser:
        """Update a user."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            return self.users_api.update_user(self.app_id, user_id, options, _headers=self.request_headers).user
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def create(self, args: CreateUserArgs) -> PassageUser:
        """Create a user."""
        if not args.email and not args.phone:
            msg = "At least one of args.email or args.phone is required."
            raise ValueError(msg)

        try:
            return self.users_api.create_user(self.app_id, args, _headers=self.request_headers).user
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def delete(self, user_id: str) -> None:
        """Delete a user using their user ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            self.users_api.delete_user(self.app_id, user_id, _headers=self.request_headers)
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def list_devices(self, user_id: str) -> list[WebAuthnDevices]:
        """Get a user's devices using their user ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            return self.user_devices_api.list_user_devices(self.app_id, user_id, _headers=self.request_headers).devices
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def revoke_device(self, user_id: str, device_id: str) -> None:
        """Revoke a user's device using their user ID and the device ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        if not device_id:
            msg = "device_id is required."
            raise ValueError(msg)

        try:
            self.user_devices_api.delete_user_devices(self.app_id, user_id, device_id, _headers=self.request_headers)
        except ApiException as e:
            raise PassageError.from_response_error(e) from e

    def revoke_refresh_tokens(self, user_id: str) -> None:
        """Revokes all of a user's Refresh Tokens using their User ID."""
        if not user_id:
            msg = "user_id is required."
            raise ValueError(msg)

        try:
            self.tokens_api.revoke_user_refresh_tokens(self.app_id, user_id, _headers=self.request_headers)
        except ApiException as e:
            raise PassageError.from_response_error(e) from e



# coding: utf-8

"""
    Passage Management API

    Passage's management API to manage your Passage apps and users.

    The version of the OpenAPI document: 1
    Contact: support@passage.id
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import Optional
from typing_extensions import Annotated
from passageidentity.openapi_client.models.create_user_args import CreateUserArgs
from passageidentity.openapi_client.models.list_paginated_users_response import ListPaginatedUsersResponse
from passageidentity.openapi_client.models.update_user_args import UpdateUserArgs
from passageidentity.openapi_client.models.user_response import UserResponse

from passageidentity.openapi_client.api_client import ApiClient, RequestSerialized
from passageidentity.openapi_client.api_response import ApiResponse
from passageidentity.openapi_client.rest import RESTResponseType


class UsersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def activate_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserResponse:
        """Activate User

        Activate a user. They will now be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._activate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def activate_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserResponse]:
        """Activate User

        Activate a user. They will now be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._activate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def activate_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Activate User

        Activate a user. They will now be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._activate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _activate_user_serialize(
        self,
        app_id,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        if user_id is not None:
            _path_params['user_id'] = user_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/apps/{app_id}/users/{user_id}/activate',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        create_user_args: Annotated[CreateUserArgs, Field(description="email, phone, user_metadata")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserResponse:
        """Create User

        Create user for an application. Must provide an email of phone number identifier.

        :param app_id: App ID (required)
        :type app_id: str
        :param create_user_args: email, phone, user_metadata (required)
        :type create_user_args: CreateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_user_serialize(
            app_id=app_id,
            create_user_args=create_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        create_user_args: Annotated[CreateUserArgs, Field(description="email, phone, user_metadata")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserResponse]:
        """Create User

        Create user for an application. Must provide an email of phone number identifier.

        :param app_id: App ID (required)
        :type app_id: str
        :param create_user_args: email, phone, user_metadata (required)
        :type create_user_args: CreateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_user_serialize(
            app_id=app_id,
            create_user_args=create_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        create_user_args: Annotated[CreateUserArgs, Field(description="email, phone, user_metadata")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create User

        Create user for an application. Must provide an email of phone number identifier.

        :param app_id: App ID (required)
        :type app_id: str
        :param create_user_args: email, phone, user_metadata (required)
        :type create_user_args: CreateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_user_serialize(
            app_id=app_id,
            create_user_args=create_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_user_serialize(
        self,
        app_id,
        create_user_args,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_user_args is not None:
            _body_params = create_user_args


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/apps/{app_id}/users',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def deactivate_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserResponse:
        """Deactivate User

        Deactivate a user. Their account will still exist, but they will not be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def deactivate_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserResponse]:
        """Deactivate User

        Deactivate a user. Their account will still exist, but they will not be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def deactivate_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deactivate User

        Deactivate a user. Their account will still exist, but they will not be able to login.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deactivate_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _deactivate_user_serialize(
        self,
        app_id,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        if user_id is not None:
            _path_params['user_id'] = user_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/apps/{app_id}/users/{user_id}/deactivate',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete User

        Delete a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete User

        Delete a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete User

        Delete a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_user_serialize(
        self,
        app_id,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        if user_id is not None:
            _path_params['user_id'] = user_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/apps/{app_id}/users/{user_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserResponse:
        """Get User

        Get information about a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserResponse]:
        """Get User

        Get information about a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get User

        Get information about a user.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_user_serialize(
            app_id=app_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_user_serialize(
        self,
        app_id,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        if user_id is not None:
            _path_params['user_id'] = user_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/apps/{app_id}/users/{user_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_paginated_users(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        page: Annotated[Optional[StrictInt], Field(description="page to fetch (min=1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="number of users to fetch per page (max=500)")] = None,
        created_before: Annotated[Optional[StrictInt], Field(description="Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`")] = None,
        identifier: Annotated[Optional[StrictStr], Field(description="search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)")] = None,
        id: Annotated[Optional[StrictStr], Field(description="search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)")] = None,
        login_count: Annotated[Optional[StrictInt], Field(description="search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)")] = None,
        created_at: Annotated[Optional[StrictStr], Field(description="search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        updated_at: Annotated[Optional[StrictStr], Field(description="search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        last_login_at: Annotated[Optional[StrictStr], Field(description="search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ListPaginatedUsersResponse:
        """List Users

        List users for an app.

        :param app_id: App ID (required)
        :type app_id: str
        :param page: page to fetch (min=1)
        :type page: int
        :param limit: number of users to fetch per page (max=500)
        :type limit: int
        :param created_before: Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)
        :type created_before: int
        :param order_by: Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`
        :type order_by: str
        :param identifier: search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)
        :type identifier: str
        :param id: search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)
        :type id: str
        :param login_count: search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)
        :type login_count: int
        :param status: search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)
        :type status: str
        :param created_at: search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type created_at: str
        :param updated_at: search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type updated_at: str
        :param last_login_at: search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type last_login_at: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_paginated_users_serialize(
            app_id=app_id,
            page=page,
            limit=limit,
            created_before=created_before,
            order_by=order_by,
            identifier=identifier,
            id=id,
            login_count=login_count,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            last_login_at=last_login_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListPaginatedUsersResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_paginated_users_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        page: Annotated[Optional[StrictInt], Field(description="page to fetch (min=1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="number of users to fetch per page (max=500)")] = None,
        created_before: Annotated[Optional[StrictInt], Field(description="Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`")] = None,
        identifier: Annotated[Optional[StrictStr], Field(description="search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)")] = None,
        id: Annotated[Optional[StrictStr], Field(description="search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)")] = None,
        login_count: Annotated[Optional[StrictInt], Field(description="search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)")] = None,
        created_at: Annotated[Optional[StrictStr], Field(description="search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        updated_at: Annotated[Optional[StrictStr], Field(description="search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        last_login_at: Annotated[Optional[StrictStr], Field(description="search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ListPaginatedUsersResponse]:
        """List Users

        List users for an app.

        :param app_id: App ID (required)
        :type app_id: str
        :param page: page to fetch (min=1)
        :type page: int
        :param limit: number of users to fetch per page (max=500)
        :type limit: int
        :param created_before: Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)
        :type created_before: int
        :param order_by: Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`
        :type order_by: str
        :param identifier: search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)
        :type identifier: str
        :param id: search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)
        :type id: str
        :param login_count: search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)
        :type login_count: int
        :param status: search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)
        :type status: str
        :param created_at: search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type created_at: str
        :param updated_at: search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type updated_at: str
        :param last_login_at: search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type last_login_at: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_paginated_users_serialize(
            app_id=app_id,
            page=page,
            limit=limit,
            created_before=created_before,
            order_by=order_by,
            identifier=identifier,
            id=id,
            login_count=login_count,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            last_login_at=last_login_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListPaginatedUsersResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_paginated_users_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        page: Annotated[Optional[StrictInt], Field(description="page to fetch (min=1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="number of users to fetch per page (max=500)")] = None,
        created_before: Annotated[Optional[StrictInt], Field(description="Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`")] = None,
        identifier: Annotated[Optional[StrictStr], Field(description="search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)")] = None,
        id: Annotated[Optional[StrictStr], Field(description="search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)")] = None,
        login_count: Annotated[Optional[StrictInt], Field(description="search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)")] = None,
        status: Annotated[Optional[StrictStr], Field(description="search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)")] = None,
        created_at: Annotated[Optional[StrictStr], Field(description="search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        updated_at: Annotated[Optional[StrictStr], Field(description="search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        last_login_at: Annotated[Optional[StrictStr], Field(description="search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Users

        List users for an app.

        :param app_id: App ID (required)
        :type app_id: str
        :param page: page to fetch (min=1)
        :type page: int
        :param limit: number of users to fetch per page (max=500)
        :type limit: int
        :param created_before: Unix timestamp to anchor pagination results (fetches events that were created before the timestamp)
        :type created_before: int
        :param order_by: Comma separated list of <field>:<ASC/DESC> (example: order_by=id:DESC,created_at:ASC) **cannot order_by `identifier`
        :type order_by: str
        :param identifier: search users email OR phone (pagination prepended operators identifier=<val>, identifier=<ne:val>, identifier=<gt:val>, identifier=<lt:val>, identifier=<like:val>, identifier=<not_like:val>)
        :type identifier: str
        :param id: search users id (pagination prepended operators id=<val>, id=<ne:val>, id=<gt:val>, id=<lt:val>, id=<like:val>, id=<not_like:val>)
        :type id: str
        :param login_count: search users login_count (pagination prepended operators login_count=<val>, login_count=<ne:val>, login_count=<gt:val>, login_count=<lt:val>)
        :type login_count: int
        :param status: search users by status (pagination prepended operators status=<val>, status=<ne:val>, status=<gt:val>, status=<lt:val>, status=<like:val>, status=<not_like:val>) -- valid values: (active, inactive, pending)
        :type status: str
        :param created_at: search users created_at (pagination prepended operators created_at=<val>, created_at=<ne:val>, created_at=<gt:val>, created_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type created_at: str
        :param updated_at: search users updated_at (pagination prepended operators updated_at=<val>, updated_at=<ne:val>, updated_at=<gt:val>, updated_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type updated_at: str
        :param last_login_at: search users last_login_at (pagination prepended operators last_login_at=<val>, lat_login_at=<ne:val>, last_login_at=<gt:val>, last_login_at=<lt:val> -- valid timestamp in the format: 2006-01-02T15:04:05.000000Z required
        :type last_login_at: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_paginated_users_serialize(
            app_id=app_id,
            page=page,
            limit=limit,
            created_before=created_before,
            order_by=order_by,
            identifier=identifier,
            id=id,
            login_count=login_count,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            last_login_at=last_login_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListPaginatedUsersResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_paginated_users_serialize(
        self,
        app_id,
        page,
        limit,
        created_before,
        order_by,
        identifier,
        id,
        login_count,
        status,
        created_at,
        updated_at,
        last_login_at,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if created_before is not None:
            
            _query_params.append(('created_before', created_before))
            
        if order_by is not None:
            
            _query_params.append(('order_by', order_by))
            
        if identifier is not None:
            
            _query_params.append(('identifier', identifier))
            
        if id is not None:
            
            _query_params.append(('id', id))
            
        if login_count is not None:
            
            _query_params.append(('login_count', login_count))
            
        if status is not None:
            
            _query_params.append(('status', status))
            
        if created_at is not None:
            
            _query_params.append(('created_at', created_at))
            
        if updated_at is not None:
            
            _query_params.append(('updated_at', updated_at))
            
        if last_login_at is not None:
            
            _query_params.append(('last_login_at', last_login_at))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/apps/{app_id}/users',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_user(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        update_user_args: Annotated[UpdateUserArgs, Field(description="user settings")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserResponse:
        """Update User

        Update a user's information.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param update_user_args: user settings (required)
        :type update_user_args: UpdateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_user_serialize(
            app_id=app_id,
            user_id=user_id,
            update_user_args=update_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_user_with_http_info(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        update_user_args: Annotated[UpdateUserArgs, Field(description="user settings")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UserResponse]:
        """Update User

        Update a user's information.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param update_user_args: user settings (required)
        :type update_user_args: UpdateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_user_serialize(
            app_id=app_id,
            user_id=user_id,
            update_user_args=update_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_user_without_preload_content(
        self,
        app_id: Annotated[StrictStr, Field(description="App ID")],
        user_id: Annotated[StrictStr, Field(description="User ID")],
        update_user_args: Annotated[UpdateUserArgs, Field(description="user settings")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update User

        Update a user's information.

        :param app_id: App ID (required)
        :type app_id: str
        :param user_id: User ID (required)
        :type user_id: str
        :param update_user_args: user settings (required)
        :type update_user_args: UpdateUserArgs
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_user_serialize(
            app_id=app_id,
            user_id=user_id,
            update_user_args=update_user_args,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserResponse",
            '400': "Model400Error",
            '401': "Model401Error",
            '404': "Model404Error",
            '500': "Model500Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_user_serialize(
        self,
        app_id,
        user_id,
        update_user_args,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if app_id is not None:
            _path_params['app_id'] = app_id
        if user_id is not None:
            _path_params['user_id'] = user_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_user_args is not None:
            _body_params = update_user_args


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/apps/{app_id}/users/{user_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





"""Provides the Passage class for interacting with the Passage API."""

from importlib import metadata

from passageidentity.auth import Auth
from passageidentity.user import User


class Passage:
    """Passage class for interacting with the Passage API."""

    def __init__(self, app_id: str, api_key: str) -> None:
        """Initialize a new Passage instance."""
        if not app_id:
            msg = "A Passage app ID is required. Please include (app_id=YOUR_APP_ID, api_key=YOUR_API_KEY)."
            raise ValueError(msg)

        if not api_key:
            msg = "A Passage API key is required. Please include (app_id=YOUR_APP_ID, api_key=YOUR_API_KEY)."
            raise ValueError(msg)

        request_headers = {
            "Authorization": f"Bearer {api_key}",
            "Passage-Version": f"passage-python {metadata.version('passage-identity')}",
        }

        self.auth = Auth(app_id, request_headers)
        self.user = User(app_id, request_headers)



"""Defines options for creating a Magic Link."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from passageidentity.openapi_client.models.magic_link_language import MagicLinkLanguage


class MagicLinkOptions:
    """Options for creating a Magic Link."""

    language: MagicLinkLanguage | None
    magic_link_path: str | None
    redirect_url: str | None
    ttl: int | None

    def __init__(
        self,
        *,
        language: MagicLinkLanguage | None = None,
        magic_link_path: str | None = None,
        redirect_url: str | None = None,
        ttl: int | None = None,
    ) -> None:
        """Initialize the options with the given values."""
        self.language = language
        self.magic_link_path = magic_link_path
        self.redirect_url = redirect_url
        self.ttl = ttl



# coding: utf-8

# flake8: noqa
"""
    Passage Management API

    Passage's management API to manage your Passage apps and users.

    The version of the OpenAPI document: 1
    Contact: support@passage.id
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from passageidentity.openapi_client.models.apple_user_social_connection import AppleUserSocialConnection
from passageidentity.openapi_client.models.create_magic_link_request import CreateMagicLinkRequest
from passageidentity.openapi_client.models.create_user_args import CreateUserArgs
from passageidentity.openapi_client.models.github_user_social_connection import GithubUserSocialConnection
from passageidentity.openapi_client.models.google_user_social_connection import GoogleUserSocialConnection
from passageidentity.openapi_client.models.link import Link
from passageidentity.openapi_client.models.list_devices_response import ListDevicesResponse
from passageidentity.openapi_client.models.list_paginated_users_item import ListPaginatedUsersItem
from passageidentity.openapi_client.models.list_paginated_users_response import ListPaginatedUsersResponse
from passageidentity.openapi_client.models.magic_link import MagicLink
from passageidentity.openapi_client.models.magic_link_channel import MagicLinkChannel
from passageidentity.openapi_client.models.magic_link_language import MagicLinkLanguage
from passageidentity.openapi_client.models.magic_link_response import MagicLinkResponse
from passageidentity.openapi_client.models.magic_link_type import MagicLinkType
from passageidentity.openapi_client.models.model400_error import Model400Error
from passageidentity.openapi_client.models.model401_error import Model401Error
from passageidentity.openapi_client.models.model403_error import Model403Error
from passageidentity.openapi_client.models.model404_error import Model404Error
from passageidentity.openapi_client.models.model500_error import Model500Error
from passageidentity.openapi_client.models.nonce import Nonce
from passageidentity.openapi_client.models.paginated_links import PaginatedLinks
from passageidentity.openapi_client.models.passage_user import PassageUser
from passageidentity.openapi_client.models.social_connection_type import SocialConnectionType
from passageidentity.openapi_client.models.update_user_args import UpdateUserArgs
from passageidentity.openapi_client.models.user_event_action import UserEventAction
from passageidentity.openapi_client.models.user_event_status import UserEventStatus
from passageidentity.openapi_client.models.user_recent_event import UserRecentEvent
from passageidentity.openapi_client.models.user_response import UserResponse
from passageidentity.openapi_client.models.user_social_connections import UserSocialConnections
from passageidentity.openapi_client.models.user_status import UserStatus
from passageidentity.openapi_client.models.web_authn_devices import WebAuthnDevices
from passageidentity.openapi_client.models.web_authn_icons import WebAuthnIcons
from passageidentity.openapi_client.models.web_authn_type import WebAuthnType



# coding: utf-8

"""
    Passage Management API

    Passage's management API to manage your Passage apps and users.

    The version of the OpenAPI document: 1
    Contact: support@passage.id
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from passageidentity.openapi_client.models.user_status import UserStatus
from typing import Optional, Set
from typing_extensions import Self

class ListPaginatedUsersItem(BaseModel):
    """
    ListPaginatedUsersItem
    """ # noqa: E501
    created_at: datetime
    email: StrictStr
    email_verified: StrictBool
    external_id: StrictStr = Field(description="The external ID of the user. Only set if the user was created in a Flex app.")
    id: StrictStr
    last_login_at: datetime
    login_count: StrictInt
    phone: StrictStr
    phone_verified: StrictBool
    status: UserStatus
    updated_at: datetime
    user_metadata: Optional[Dict[str, Any]]
    __properties: ClassVar[List[str]] = ["created_at", "email", "email_verified", "external_id", "id", "last_login_at", "login_count", "phone", "phone_verified", "status", "updated_at", "user_metadata"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ListPaginatedUsersItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if user_metadata (nullable) is None
        # and model_fields_set contains the field
        if self.user_metadata is None and "user_metadata" in self.model_fields_set:
            _dict['user_metadata'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ListPaginatedUsersItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "created_at": obj.get("created_at"),
            "email": obj.get("email"),
            "email_verified": obj.get("email_verified"),
            "external_id": obj.get("external_id"),
            "id": obj.get("id"),
            "last_login_at": obj.get("last_login_at"),
            "login_count": obj.get("login_count"),
            "phone": obj.get("phone"),
            "phone_verified": obj.get("phone_verified"),
            "status": obj.get("status"),
            "updated_at": obj.get("updated_at"),
            "user_metadata": obj.get("user_metadata")
        })
        return _obj





"""Provides the Auth class for interacting with the Passage API."""

from __future__ import annotations

from typing import TYPE_CHECKING

import jwt as pyjwt

from passageidentity.errors import PassageError
from passageidentity.models.magic_link_args import MagicLinkWithEmailArgs, MagicLinkWithPhoneArgs, MagicLinkWithUserArgs
from passageidentity.openapi_client.api.magic_links_api import MagicLinksApi
from passageidentity.openapi_client.exceptions import ApiException
from passageidentity.openapi_client.models.create_magic_link_request import CreateMagicLinkRequest
from passageidentity.openapi_client.models.magic_link_channel import MagicLinkChannel

if TYPE_CHECKING:
    from passageidentity.models.magic_link_args import MagicLinkArgs
    from passageidentity.models.magic_link_options import MagicLinkOptions
    from passageidentity.openapi_client.models.magic_link import MagicLink


class Auth:
    """Auth class for handling operations to authenticate and validate JWTs."""

    def __init__(self, app_id: str, request_headers: dict[str, str]) -> None:
        """Initialize the Auth class with the app ID and request headers."""
        self.app_id = app_id
        self.request_headers = request_headers
        self.jwks = pyjwt.PyJWKClient(
            f"https://auth.passage.id/v1/apps/{self.app_id}/.well-known/jwks.json",
            # must set a user agent to avoid 403 from CF
            headers={"User-Agent": "passageidentity/python"},
        )

        self.magic_links_api = MagicLinksApi()

    def validate_jwt(self, jwt: str) -> str:
        """Verify the JWT and return the user ID for the authenticated user, or throw a PassageError."""
        if not jwt:
            msg = "jwt is required."
            raise ValueError(msg)

        header = pyjwt.get_unverified_header(jwt)
        kid = header.get("kid")

        if kid is None:
            msg = "kid is missing in the JWT header."
            raise ValueError(msg)

        public_key = self.jwks.get_signing_key(kid)
        claims = pyjwt.decode(
            jwt,
            public_key,
            audience=self.app_id,
            algorithms=["RS256"],
        )

        sub = claims.get("sub")
        if sub is None:
            msg = "sub is missing in the JWT claims."
            raise ValueError(msg)

        return sub

    def create_magic_link(self, args: MagicLinkArgs, options: MagicLinkOptions | None = None) -> MagicLink:
        """Create a Magic Link for your app."""
        payload = CreateMagicLinkRequest()
        payload.type = args.type
        payload.send = args.send

        if isinstance(args, MagicLinkWithEmailArgs):
            payload.email = args.email
            payload.channel = MagicLinkChannel.EMAIL
        elif isinstance(args, MagicLinkWithPhoneArgs):
            payload.phone = args.phone
            payload.channel = MagicLinkChannel.PHONE
        elif isinstance(args, MagicLinkWithUserArgs):
            payload.user_id = args.user_id
            payload.channel = args.channel
        else:
            msg = "args must be an instance of MagicLinkArgs"
            raise TypeError(msg)

        if options:
            payload.language = options.language
            payload.magic_link_path = options.magic_link_path
            payload.redirect_url = options.redirect_url
            payload.ttl = options.ttl

        try:
            return self.magic_links_api.create_magic_link(
                self.app_id,
                payload,
                _headers=self.request_headers,
            ).magic_link
        except ApiException as e:
            raise PassageError.from_response_error(e) from e



"""Defines custom error classes for handling Passage-related errors."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from passageidentity.openapi_client.exceptions import ApiException


class PassageError(Exception):
    """Error class for handling Passage errors."""

    message: str
    status_code: int | None
    error_code: str | None

    def __str__(self) -> str:
        """Return the error message."""
        return self.message

    @classmethod
    def from_response_error(cls, response_error: ApiException) -> PassageError:
        """Initialize the error with a response body and optional message."""
        if response_error.data is not None:
            data_dict = response_error.data.to_dict()
            error_code = data_dict.get("code")
            error_msg = data_dict.get("error")
        else:
            error_code = None
            error_msg = str(response_error.body)

        psg_error = cls()
        psg_error.message = error_msg
        psg_error.status_code = response_error.status
        psg_error.error_code = error_code

        return psg_error



"""Initializes the Passage identity package."""

from .errors import PassageError
from .models import (
    MagicLinkArgs,
    MagicLinkOptions,
    MagicLinkWithEmailArgs,
    MagicLinkWithPhoneArgs,
    MagicLinkWithUserArgs,
)
from .openapi_client.models import (
    CreateUserArgs,
    MagicLink,
    MagicLinkChannel,
    MagicLinkLanguage,
    MagicLinkType,
    PassageUser,
    UpdateUserArgs,
    UserStatus,
    WebAuthnDevices,
    WebAuthnType,
)
from .passage import Passage

__all__ = [
    "CreateUserArgs",
    "MagicLink",
    "MagicLinkArgs",
    "MagicLinkChannel",
    "MagicLinkLanguage",
    "MagicLinkOptions",
    "MagicLinkType",
    "MagicLinkWithEmailArgs",
    "MagicLinkWithPhoneArgs",
    "MagicLinkWithUserArgs",
    "Passage",
    "PassageError",
    "PassageUser",
    "UpdateUserArgs",
    "UserStatus",
    "WebAuthnDevices",
    "WebAuthnType",
]



"""Defines required arguments for creating a Magic Link."""

from typing import Union

from passageidentity.openapi_client.models.magic_link_channel import MagicLinkChannel
from passageidentity.openapi_client.models.magic_link_type import MagicLinkType


class MagicLinkArgsBase:
    """Base class for MagicLinkArgs."""

    type: MagicLinkType
    send: bool


class MagicLinkWithEmailArgs(MagicLinkArgsBase):
    """Arguments for creating a Magic Link with an email."""

    email: str

    def __init__(self, *, email: str, link_type: MagicLinkType, send: bool) -> None:
        """Initialize the MagicLinkWithEmailArgs with the email."""
        self.email = email
        self.type = link_type
        self.send = send


class MagicLinkWithPhoneArgs(MagicLinkArgsBase):
    """Arguments for creating a Magic Link with a phone number."""

    phone: str

    def __init__(self, *, phone: str, link_type: MagicLinkType, send: bool) -> None:
        """Initialize the MagicLinkWithPhoneArgs with the phone number."""
        self.phone = phone
        self.type = link_type
        self.send = send


class MagicLinkWithUserArgs(MagicLinkArgsBase):
    """Arguments for creating a Magic Link with a user ID."""

    user_id: str
    channel: MagicLinkChannel

    def __init__(self, *, user_id: str, channel: MagicLinkChannel, link_type: MagicLinkType, send: bool) -> None:
        """Initialize the MagicLinkWithUserArgs with the user ID."""
        self.user_id = user_id
        self.channel = channel
        self.type = link_type
        self.send = send


MagicLinkArgs = Union[MagicLinkWithEmailArgs, MagicLinkWithPhoneArgs, MagicLinkWithUserArgs]



# coding: utf-8

"""
    Passage Management API

    Passage's management API to manage your Passage apps and users.

    The version of the OpenAPI document: 1
    Contact: support@passage.id
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class UserEventStatus(str, Enum):
    """
    UserEventStatus
    """

    """
    allowed enum values
    """
    COMPLETE = 'complete'
    INCOMPLETE = 'incomplete'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of UserEventStatus from a JSON string"""
        return cls(json.loads(json_str))


